var MeeboBrowser = function() {

    // #region Private Variables

    var buddyListId = 'buddylistwin';

    var backgroundObjects = [
        'meebologo',
        'meebolinks',
        'meeboconsole',
        'slogan',
        'languagelinks',
        'copyright',
        'accounts-console',
        'prefs-console',
        'widgets-console'
        ]

    var consoleObjects = [
        '~meebologo',
        '~slogan',
        'meebolinks',
        'innermeebolinks',
        'meeboconsole',
        'copyright'
        ]

    var lastDetectedBgColor = null;
    var previousUnload = null;
    var newUnload = null;
    var backgroundTimer = null;
    var titleHistory = Array();
    var titleIndex = 0;
    var previousTitle = null;

    // #endregion Private Variables

    // #region Private Methods

    var nudgeColor = function(c) {
        if ((typeof(c) == 'string') &&
            ((c.length == 4) || (c.length == 7)) &&
            (c.charAt(0) == '#')) {
            if (c.length == 4) c = '#' +
                c.charAt(1) + c.charAt(1) +
                c.charAt(2) + c.charAt(2) +
                c.charAt(3) + c.charAt(3);
            var cxor1 = function(i) {
                var code = c.charAt(i).charCodeAt();
                if (code > 62) code--;
                code = code ^ 1;
                if (code > 62) code++;
                return String.fromCharCode(code);
            };
            c = '#' +
                c.charAt(1) + cxor1(2) +
                c.charAt(3) + cxor1(4) +
                c.charAt(5) + cxor1(6);
        }
        return c;
    };

    var setConsoleVisibility = function(state) {
        var display = state ? '' : 'none';
        var visibility = state ? '' : 'hidden';
        for (var c = 0; c < consoleObjects.length; c++)
        {
            var id = consoleObjects[c];
            var hide = (id.length > 0) && (id.charAt(0) == '~');
            if (hide) id = id.substr(1);
            var element = document.getElementById(id);
            if (element)
            {
                if (hide)
                    element.style.visibility = visibility;
                else
                    element.style.display = display;
            }
        }
    };

    var arrayIndexOf = function(arr, el, start) {
        var start = start || 0;
        for ( var i=0; i < arr.length; ++i ) {
            if ( arr[i] === el ) {
                return i;
            }
        }
        return -1;
    };

    var loadScript = function(scriptURL) {
        var scriptElem = document.createElement('SCRIPT');
        scriptElem.setAttribute('language', 'JavaScript');
        scriptElem.setAttribute('src', scriptURL);
        document.body.appendChild(scriptElem);
    };

    var hookUnload = function(ourUnload) {
        previousUnload = document.body.onunload;
        newUnload = (previousUnload != null) ?
            function() { previousUnload(); ourUnload(); } :
            ourUnload;
        document.body.onunload = newUnload;
    };

    var unhookUnload = function() {
        if (document.body.onunload == newUnload)
            document.body.onunload = previousUnload;
        else if ((document.body.onunload != null) && (document.body.onunload != previousUnload))
            alert('Sorry, I could not unhook the unload event, because it was changed since I set it.');
    };

    // #endregion Private Methods

    // #region Public Methods

    return {

        SetBuddyListVisibility: function(state) {
            var buddylistwin = document.getElementById(buddyListId);
            if (buddylistwin != null)
            {
                buddylistwin.style.display =
                    state ? '' :
                    buddylistwin.style.display != '' ? '' :
                    'none';
            }
        },

        DocumentTitleChanged: function() {
            var title = document.title;
            var popup = false;
            if (titleHistory[titleIndex] == title)
            {
                // we found same the same string where we saw it last time
                //  nothing to see here, move along
            }
            else
            {
                var titlePos = arrayIndexOf(titleHistory, title);
                if (titlePos > -1)
                {
                    // damn we've seen this title before, but we're now in the wrong place
                    // reset the position so that stuff works
                    titleIndex = titlePos;
                }
                else
                {
                    // brand new title, great
                    titleHistory[titleIndex] = title;
                    popup = true;
                }
                titleHistory.length = titleIndex + 1;
            }
            // get ready for the next title
            if (title.length > 0) titleIndex = (titleIndex + 1) % 3;
            var displayTitle = titleHistory.join('\n');

            if (previousTitle != displayTitle)
            {
                previousTitle = displayTitle;
                external.NotifyIconText = displayTitle;
                if (popup && displayTitle && (displayTitle.length > 0)) {
                    external.ShowBalloonTip(10000, 'Meebo', displayTitle, 'Info');
                }
            }
        },

        FixBackgroundColors: function() {
            var bgColor = document.body.currentStyle.backgroundColor;
            if (lastDetectedBgColor != bgColor)
            {
                lastDetectedBgColor = bgColor;
                external.TransparentColor = bgColor;
                var newColor = nudgeColor(bgColor);
                for (var c = 0; c < backgroundObjects.length; c++)
                {
                    var element = document.getElementById(backgroundObjects[c]);
                    if (element) element.style.background = newColor;
                }
            }
        },

        MenuItem_ShowConsole_Click: function(checked) {
            external.SetMenuItemChecked('ShowConsole', checked);
            external.SetSetting('ShowConsole', checked)
            setConsoleVisibility(checked);
        },

        MenuItem_TopMost_Click: function(checked) {
            external.SetMenuItemChecked('TopMost', checked);
            external.SetSetting('TopMost', checked)
            external.TopMost = checked;
        },

        MenuItem_JsShell_Click: function() {
            loadScript(document.MeeboBrowserScriptUri + 'jsshell.js?' + (new Date()).getTime());
        },

        MenuItem_Reload_Click: function() {
            unhookUnload();
            this.Unload();
            loadScript(document.MeeboBrowserScriptUri + 'meebobrowser.js?' + (new Date()).getTime());
        },

        Load: function() {
            external.AddMenuItem('ShowConsole', 'Show &Console');
            external.AddMenuItem('TopMost', 'Always On &Top');
            external.AddMenuItem('Separator1', '-');
            external.AddMenuItem('Reload', '&Reload JavaScript');
            external.AddMenuItem('JsShell', '&JavaScript Shell');
            external.AddMenuItem('Separator2', '-');
            var showConsole = external.GetSetting('ShowConsole').toLowerCase() == 'true';
            external.SetMenuItemChecked('ShowConsole', showConsole);
            setConsoleVisibility(showConsole);
            this.MenuItem_TopMost_Click(external.GetSetting('TopMost').toLowerCase() == 'true');

            this.FixBackgroundColors();
            backgroundTimer = setInterval('MeeboBrowser.FixBackgroundColors()', 200);
            document.body.style.backgroundImage = 'url()';
            var languagelinks = document.getElementById('languagelinks');
            if ((languagelinks != null) && (languagelinks.style.paddingTop != '0px')) {
                languagelinks.style.marginTop = languagelinks.currentStyle.paddingTop;
                languagelinks.style.paddingTop = '0px';
            }
            hookUnload(this.Unload);
            external.SplashPanelVisible = false;
        },

        Unload: function() {
            external.ClearMenuItems();
            setConsoleVisibility(true);
            external.TransparentColor = '#fefeff';
            clearInterval(backgroundTimer);
            external.SplashPanelVisible = true;
        }
    };

    // #endregion Public Methods

}();
MeeboBrowser.Load();
